@@ -1,6 +1,7 @@
 use std::cell::RefCell;
 use std::ops::{Deref, DerefMut};
 
+use crate::arg::ArgStream;
 use crate::err::Result;
 use crate::opt::OptCallback;
 use crate::parser::HashMapIter;
@@ -20,6 +21,21 @@ impl<S: Set + Default, P: Parser + Default> SingleApp<S, P> {
         Self { name, set, parser }
     }
 
+    pub fn with_name(mut self, name: String) -> Self {
+        self.name = name;
+        self
+    }
+
+    pub fn with_optset(mut self, set: S) -> Self {
+        self.set = set;
+        self
+    }
+
+    pub fn with_parser(mut self, parser: P) -> Self {
+        self.parser = parser;
+        self
+    }
+
     pub fn get_name(&self) -> &str {
         self.name.as_str()
     }
@@ -58,12 +74,12 @@ impl<S: Set + Default, P: Parser + Default> SingleApp<S, P> {
 
     pub fn run<RET, F: FnMut(bool, SingleApp<S, P>) -> Result<RET>>(
         &mut self,
-        mut iter: impl Iterator<Item = String>,
+        iter: impl Iterator<Item = String>,
         mut r: F,
     ) -> Result<RET> {
         let set = &mut self.set;
         let parser = &mut self.parser;
-        let ret = parser.parse(set, &mut iter)?;
+        let ret = parser.parse(set, &mut ArgStream::from(iter))?;
         let _self = std::mem::take(self);
 
         r(ret, _self)
