@@ -13,6 +13,7 @@ pub(crate) mod pat;
 #[macro_use]
 extern crate tracing;
 
+use crate::arg::ArgStream;
 use crate::err::Result;
 use crate::parser::Parser;
 use crate::set::Set;
@@ -26,7 +27,7 @@ pub fn gstr(s: &str) -> Ustr {
     Ustr::from(s)
 }
 
-pub struct ReturnValue<'a, 'b>(&'b mut dyn Parser, &'a mut dyn Set);
+pub struct ReturnValue<'a, 'b>(pub &'b mut dyn Parser, pub &'a mut dyn Set);
 
 pub fn getopt_impl<'a, 'b>(
     iter: impl Iterator<Item = String>,
@@ -40,7 +41,9 @@ pub fn getopt_impl<'a, 'b>(
     let mut index = 0;
 
     for (parser, set) in parsers.into_iter().zip(sets.into_iter()) {
-        match parser.parse(set, &mut args.iter().map(|v| v.clone())) {
+        let mut stream = ArgStream::from(args.clone().into_iter());
+
+        match parser.parse(set, &mut stream) {
             Ok(rv) => {
                 if rv {
                     return Ok(Some(ReturnValue(parser, set)));
@@ -60,11 +63,13 @@ pub fn getopt_impl<'a, 'b>(
 }
 
 pub fn getopt_impl_s<'a, 'b>(
-    mut iter: impl Iterator<Item = String>,
+    iter: impl Iterator<Item = String>,
     set: &'a mut dyn Set,
     parser: &'b mut dyn Parser,
 ) -> Result<Option<ReturnValue<'a, 'b>>> {
-    if parser.parse(set, &mut iter)? {
+    let mut stream = ArgStream::from(iter);
+
+    if parser.parse(set, &mut stream)? {
         return Ok(Some(ReturnValue(parser, set)));
     } else {
         Ok(None)
@@ -90,27 +95,6 @@ macro_rules! getopt {
 }
 
 pub mod tools {
-    use crate::opt::{ArrayCreator, BoolCreator, FltCreator, IntCreator, StrCreator, UintCreator};
-    use crate::opt::{CmdCreator, MainCreator, PosCreator};
-    use crate::set::Set;
-
-    pub fn initialize_creator<S: Set>(set: &mut S) {
-        set.add_creator(Box::new(ArrayCreator::default()));
-        set.add_creator(Box::new(BoolCreator::default()));
-        set.add_creator(Box::new(FltCreator::default()));
-        set.add_creator(Box::new(IntCreator::default()));
-        set.add_creator(Box::new(StrCreator::default()));
-        set.add_creator(Box::new(UintCreator::default()));
-        set.add_creator(Box::new(CmdCreator::default()));
-        set.add_creator(Box::new(MainCreator::default()));
-        set.add_creator(Box::new(PosCreator::default()));
-    }
-
-    pub fn initialize_prefix<S: Set>(set: &mut S) {
-        set.add_prefix("--".into());
-        set.add_prefix("-".into());
-    }
-
     #[macro_export]
     macro_rules! simple_main_cb {
         ($block:expr) => {
