@@ -8,7 +8,8 @@ use super::OptValueKeeper;
 use super::Parser;
 use super::ParserState;
 
-use crate::arg::ArgStream;
+use crate::arg::Argument;
+use crate::err;
 use crate::err::Result;
 use crate::opt::{OptCallback, OptValue, Style};
 use crate::proc::{Info, Matcher, NonOptMatcher, OptMatcher, Proc};
@@ -16,7 +17,7 @@ use crate::set::{OptionInfo, Set};
 use crate::uid::{Generator, Uid};
 use crate::Ustr;
 
-#[derive(Debug, Default)]
+#[derive(Debug)]
 pub struct DelayParser<G>
 where
     G: Generator + Debug + Default,
@@ -30,6 +31,24 @@ where
     noa: Vec<Ustr>,
 
     value_keeper: HashMap<Uid, Vec<OptValueKeeper>>,
+
+    strict: bool,
+}
+
+impl<G> Default for DelayParser<G>
+where
+    G: Generator + Debug + Default,
+{
+    fn default() -> Self {
+        Self {
+            uid_gen: G::default(),
+            subscriber_info: vec![],
+            callback: HashMap::default(),
+            noa: vec![],
+            value_keeper: HashMap::default(),
+            strict: true,
+        }
+    }
 }
 
 impl<G> DelayParser<G>
@@ -46,20 +65,27 @@ where
     pub fn add_delay_value(&mut self, uid: Uid, value: OptValueKeeper) {
         self.value_keeper.entry(uid).or_insert(vec![]).push(value);
     }
+
+    pub fn with_strict(mut self, strict: bool) -> Self {
+        self.strict = strict;
+        self
+    }
+
+    pub fn set_strict(&mut self, strict: bool) {
+        self.strict = strict;
+    }
 }
 
 impl<G> Parser for DelayParser<G>
 where
     G: Generator + Debug + Default,
 {
-    fn parse<'a>(
+    fn parse(
         &mut self,
-        set: &'a mut dyn Set,
-        iter: &mut dyn Iterator<Item = String>,
+        set: &mut dyn Set,
+        iter: &mut dyn Iterator<Item = Argument>,
     ) -> Result<bool> {
-        let mut argstream = ArgStream::from(iter);
         let set = set;
-        let mut iter = argstream.iter_mut();
 
         // copy the prefix, so we don't need borrow set
         let prefix: Vec<Ustr> = set.get_prefix().iter().map(|v| v.clone()).collect();
@@ -86,7 +112,7 @@ where
         // iterate the Arguments, generate option context
         // send it to Publisher
         info!("start process option ...");
-        while let Some(arg) = iter.next() {
+        while let Some(mut arg) = iter.next() {
             let mut matched = false;
             let mut consume = false;
 
@@ -95,7 +121,7 @@ where
                 if ret {
                     debug!(?arg, "after parsing ...");
                     for gen_style in &parser_state {
-                        if let Some(ret) = gen_style.gen_opt::<OptMatcher>(arg)? {
+                        if let Some(ret) = gen_style.gen_opt::<OptMatcher>(&arg)? {
                             let mut proc = ret;
 
                             if self.process(&mut proc, set)? {
@@ -111,6 +137,16 @@ where
                             }
                         }
                     }
+                    if !matched {
+                        // if current ARG is like an option, but it not matched
+                        if self.strict {
+                            return Err(err::SpecialError::InvalidOptionName(format!(
+                                "{}",
+                                arg.current.unwrap_or_default().as_str()
+                            ))
+                            .into());
+                        }
+                    }
                 }
             }
             if matched && consume {
