@@ -8,7 +8,7 @@ use super::OptValueKeeper;
 use super::Parser;
 use super::ParserState;
 
-use crate::arg::ArgStream;
+use crate::arg::Argument;
 use crate::err::Result;
 use crate::opt::{OptCallback, OptValue, Style};
 use crate::proc::{Info, Matcher, NonOptMatcher, OptMatcher, Proc};
@@ -46,14 +46,12 @@ impl<G> Parser for PreParser<G>
 where
     G: Generator + Debug + Default,
 {
-    fn parse<'a>(
+    fn parse(
         &mut self,
-        set: &'a mut dyn Set,
-        iter: &mut dyn Iterator<Item = String>,
+        set: &mut dyn Set,
+        iter: &mut dyn Iterator<Item = Argument>,
     ) -> Result<bool> {
-        let mut argstream = ArgStream::from(iter);
         let set = set;
-        let mut iter = argstream.iter_mut();
 
         // copy the prefix, so we don't need borrow set
         let prefix: Vec<Ustr> = set.get_prefix().iter().map(|v| v.clone()).collect();
@@ -80,7 +78,7 @@ where
         // iterate the Arguments, generate option context
         // send it to Publisher
         info!("start process option ...");
-        while let Some(arg) = iter.next() {
+        while let Some(mut arg) = iter.next() {
             let mut matched = false;
             let mut consume = false;
 
@@ -89,7 +87,7 @@ where
                 if ret {
                     debug!(?arg, "after parsing ...");
                     for gen_style in &parser_state {
-                        if let Some(ret) = gen_style.gen_opt::<OptMatcher>(arg)? {
+                        if let Some(ret) = gen_style.gen_opt::<OptMatcher>(&arg)? {
                             let mut proc = ret;
 
                             if self.process(&mut proc, set)? {
